package eu.eudat.models.validators;

import eu.eudat.data.query.items.table.project.ProjectTableRequest;
import eu.eudat.models.validators.fluentvalidator.FluentValidator;
import org.springframework.stereotype.Component;
import org.springframework.validation.Errors;


@Component("projectTableRequestValidator")
public class ProjectTableRequestValidator extends FluentValidator<ProjectTableRequest> {

    public ProjectTableRequestValidator() {
        ruleFor(x -> x.getOffset()).compareAs(x -> 0, (leftItem, rightItem) -> leftItem < rightItem)
                .withName("offset").withMessage("projectTableRequest.offset.negative");

        ruleFor(x -> x.getLength()).compareAs(x -> 0, (leftItem, rightItem) -> leftItem < rightItem)
                .withName("length").withMessage("projectTableRequest.length.negative");

        ruleFor(x -> x.getCriteria().getPeriodStart())
                .compareAs(x -> x.getCriteria().getPeriodEnd(), (leftItem, rightItem) -> rightItem.before(leftItem))
                .withName("criteria.periodStart").withMessage("projectTableRequest.periodStart.overlapping")
                .ruleIf(x -> x.getCriteria() != null && x.getCriteria().getPeriodStart() != null && x.getCriteria().getPeriodEnd() != null);
    }

    @Override
    public boolean supports(Class<?> aClass) {
        return ProjectTableRequest.class.equals(aClass);
    }

    public static boolean supportsType(Class<?> clazz) {
        return ProjectTableRequest.class.equals(clazz);
    }
}
