import { Injectable } from '@angular/core';
import { HttpHeaders } from '@angular/common/http';
import { BaseHttpService } from '../../utilities/cite-http-service-module/base-http.service';
import { DataTableRequest } from '../../models/data-table/DataTableRequest';
import { DataTableData } from '../../models/data-table/DataTableData';
import { Observable } from 'rxjs';
import { DataManagementPlanProfileCriteria } from '../../models/criteria/dmp-profile/DataManagementPlanProfileCriteria';
import { DataManagementPlanProfileListingModel } from '../../models/data-management-plan-profile/DataManagementPlanProfileListingModel';
import { DataManagementPlanProfileModel } from '../../models/data-management-plan-profile/DataManagementPlanProfileModel';
import { RequestItem } from '../../models/criteria/RequestItem';
import { environment } from '../../../environments/environment';

@Injectable()
export class DataManagementPlanProfileService {

	private actionUrl: string;
	private headers: HttpHeaders;

	constructor(private http: BaseHttpService) {

		this.actionUrl = environment.Server + 'dmpprofile/';

		this.headers = new HttpHeaders();
		this.headers = this.headers.set('Content-Type', 'application/json');
		this.headers = this.headers.set('Accept', 'application/json');
	}

	getPaged(dataTableRequest: DataTableRequest<DataManagementPlanProfileCriteria>): Observable<DataTableData<DataManagementPlanProfileListingModel>> {
		return this.http.post<DataTableData<DataManagementPlanProfileListingModel>>(this.actionUrl + 'getPaged', dataTableRequest, { headers: this.headers });
	}

	getSingle(id: String): Observable<DataManagementPlanProfileModel> {
		return this.http.get<DataManagementPlanProfileModel>(this.actionUrl + 'getSingle/' + id, { headers: this.headers });
	}

	createDataManagementPlan(dataManagementPlanModel: DataManagementPlanProfileModel): Observable<DataManagementPlanProfileModel> {
		return this.http.post<DataManagementPlanProfileModel>(this.actionUrl + 'createOrUpdate', dataManagementPlanModel, { headers: this.headers });
	}

	getAll(criteria: RequestItem<DataManagementPlanProfileCriteria>): Observable<DataManagementPlanProfileListingModel> {
		return this.http.post<DataManagementPlanProfileModel>(this.actionUrl + 'get/', criteria, { headers: this.headers });
	}
}
