import { HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { environment } from '../../../environments/environment';
import { ExternalDatasetCriteria } from '../../models/criteria/external-dataset/ExternalDatasetCriteria';
import { RequestItem } from '../../models/criteria/RequestItem';
import { DataTableData } from '../../models/data-table/DataTableData';
import { DataTableRequest } from '../../models/data-table/DataTableRequest';
import { ExternalDatasetModel } from '../../models/external-dataset/ExternalDatasetModel';
import { BaseHttpService } from '../../utilities/cite-http-service-module/base-http.service';


@Injectable()
export class ExternalDatasetService {

	private actionUrl: string;
	private headers: HttpHeaders;

	constructor(private http: BaseHttpService) {

		this.actionUrl = environment.Server + 'externaldatasets/';

		this.headers = new HttpHeaders();
		this.headers = this.headers.set('Content-Type', 'application/json');
		this.headers = this.headers.set('Accept', 'application/json');
	}

	getPaged(dataTableRequest: DataTableRequest<ExternalDatasetCriteria>): Observable<DataTableData<ExternalDatasetModel>> {
		return this.http.post<DataTableData<ExternalDatasetModel>>(this.actionUrl + 'getPaged', dataTableRequest, { headers: this.headers });
	}

	/* get(requestItem: RequestItem<ProjectCriteria>): Observable<ProjectModel[]> {
		 return this.http.post<ProjectModel[]>(this.actionUrl + 'get', requestItem, { headers: this.headers });
	 }*/

	getWithExternal(requestItem: RequestItem<ExternalDatasetCriteria>): Observable<ExternalDatasetModel[]> {
		return this.http.post<ExternalDatasetModel[]>(this.actionUrl + 'getWithExternal', requestItem, { headers: this.headers });
	}

	getSingle(id: string): Observable<ExternalDatasetModel> {
		return this.http.get<ExternalDatasetModel>(this.actionUrl + 'getSingle/' + id, { headers: this.headers });
	}

	create(externalDatasetModel: ExternalDatasetModel): Observable<ExternalDatasetModel> {
		return this.http.post<ExternalDatasetModel>(this.actionUrl + 'create', externalDatasetModel, { headers: this.headers });
	}
}
