import { HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import 'rxjs/add/operator/map';
import { environment } from '../../../environments/environment';
import { UserCriteria } from '../../models/criteria/users/UserCriteria';
import { DataTableData } from '../../models/data-table/DataTableData';
import { DataTableRequest } from '../../models/data-table/DataTableRequest';
import { UserListingModel } from '../../models/users/UserListingModel';
import { BaseHttpService } from '../../utilities/cite-http-service-module/base-http.service';

@Injectable()
export class UserReferenceService {

	private actionUrl: string;
	private headers: HttpHeaders;

	constructor(private http: BaseHttpService) {

		this.actionUrl = environment.Server + '/user/';

		this.headers = new HttpHeaders();
		this.headers = this.headers.set('Content-Type', 'application/json');
		this.headers = this.headers.set('Accept', 'application/json');
	}

	// formatItem(utilities: Utilities, item: ConfigurationModel): ConfigurationModel {
	//    item.validFrom = new Date(item.validFrom);
	//    item.validTo = new Date(item.validTo);
	//    return item
	// }

	getPaged(dataTableRequest: DataTableRequest<UserCriteria>): Observable<DataTableData<UserListingModel>> {
		return this.http.post<DataTableData<UserListingModel>>(this.actionUrl + 'getPaged', JSON.stringify(dataTableRequest), { headers: this.headers });
	}

	getUser(id: String): Observable<UserListingModel> {
		return this.http.get<UserListingModel>(this.actionUrl + id, { headers: this.headers });
	}

	updateRoles(itemToUpdate: UserListingModel): Observable<UserListingModel> {
		return this.http.post<UserListingModel>(this.actionUrl + 'updateRoles', JSON.stringify(itemToUpdate), { headers: this.headers });
	}

	delete(id: String): Observable<any> {
		return this.http.delete<any>(this.actionUrl + id, { headers: this.headers });
	}

	getRecentActivity(): Observable<any[]> {
		return this.http.get<any[]>(this.actionUrl + 'recentActivity', { headers: this.headers });
	}

	updateUserSettings(value: any): Observable<any[]> {
		return this.http.post<any[]>(this.actionUrl + 'settings', value, { headers: this.headers });
	}
}
