import { TranslateService } from '@ngx-translate/core';
import { Component, OnInit, Input } from '@angular/core';
import { FormControl, FormGroup, FormBuilder, Validators } from '@angular/forms';
import { BaseCriteriaComponent } from '../base/base-criteria.component';
import { ValidationContext, Validation } from '../../../../utilities/validators/ValidationContext';
import { BackendErrorValidator } from '../../../../utilities/validators/BackendErrorValidator';
import { DataManagementPlanCriteria } from '../../../../models/criteria/data-management-plan/DataManagementPlanCriteria';
import { DataManagementPlanCriteriaErrorModel } from '../../../../models/criteria/data-management-plan/DataManagementPlanCriteriaErrorModel';
import { ProjectModel } from '../../../../models/projects/ProjectModel';
import { ProjectCriteria } from '../../../../models/criteria/project/ProjectCriteria';
import { RequestItem } from '../../../../models/criteria/RequestItem';
import { create } from 'domain';
import { DataManagementPlanProfileCriteria } from '../../../../models/criteria/dmp-profile/DataManagementPlanProfileCriteria';
import { DataManagementPlanProfileCriteriaErrorModel } from '../../../../models/criteria/dmp-profile/DataManagementPlanProfileCriteriaErrorModel';

@Component({
	selector: 'app-dmp-profile-criteria-component',
	templateUrl: './dmp-profile-criteria.component.html',
	styleUrls: ['./dmp-profile-criteria.component.scss'],
})

export class DataManagementPlanProfileCriteriaComponent extends BaseCriteriaComponent implements OnInit {

	@Input()
	showProject: boolean;
	public criteria: DataManagementPlanProfileCriteria = new DataManagementPlanProfileCriteria();
	filteringProjectsAsync = false;
	filteredProjects: ProjectModel[];

	constructor(
		public language: TranslateService,
		public formBuilder: FormBuilder
	) {
		super(new DataManagementPlanProfileCriteriaErrorModel());
	}

	ngOnInit() {
		super.ngOnInit();
		if (this.criteria == null) { this.criteria = new DataManagementPlanCriteria(); }
	}

	setCriteria(criteria: DataManagementPlanProfileCriteria): void {
		this.criteria = criteria;
	}

	onCallbackError(error: any) {
		this.setErrorModel(error.error);
	}

	controlModified(): void {
		this.clearErrorModel();
		if (this.refreshCallback != null &&
			(this.criteria.like == null || this.criteria.like.length === 0 || this.criteria.like.length > 2)
		) {
			this.refreshCallback();
		}
	}
}
