import { Credential } from '../../models/login/Credential';
import { LoginOptions } from '../utilties/LoginOptions';
import { LoginService } from '../utilties/login-service';
import { HttpClient } from '@angular/common/http';
import { Component, OnInit, ElementRef, AfterViewInit, VERSION, Injectable, NgZone } from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';
import { MatPaginator, MatSort, MatSnackBar } from '@angular/material';
import { TranslateService } from '@ngx-translate/core';



@Component({
	selector: 'app-login',
	templateUrl: './login.component.html',
	styleUrls: ['./login.component.scss']
})
export class LoginComponent implements AfterViewInit {

	public auth2: any;
	public credential: Credential = new Credential();

	constructor(
		private loginService: LoginService
	) {
	}

	ngAfterViewInit() {
		this.loginService.initProviders();
	}

	public facebookLogin() {
		this.loginService.facebookLogin();
	}

	public linkedInLogin() {
		this.loginService.linkedInInitialiseLogin();
	}

	public twitterLogin() {
		this.loginService.twitterInitialiseLogin();
	}

	public nativeLogin() {
		this.loginService.nativeLogin(this.credential);
	}

	public b2AccessLogin() {
		return this.loginService.b2AccessInitialiseLogin();
	}

	public hasFacebookOauth(): boolean {
		return this.loginService.hasProvider(LoginOptions.facebookOauth);
	}

	public hasLinkedInOauth(): boolean {
		return this.loginService.hasProvider(LoginOptions.linkedInOauth);
	}

	public hasTwitterOauth(): boolean {
		return this.loginService.hasProvider(LoginOptions.twitterOauth);
	}

	public hasGoogleOauth(): boolean {
		return this.loginService.hasProvider(LoginOptions.googleOauth);
	}

	public hasNativeLogin(): boolean {
		return this.loginService.hasProvider(LoginOptions.nativeLogin);
	}

	public hasB2AccessOauth(): boolean {
		return this.loginService.hasProvider(LoginOptions.b2Access);
	}

}
